@@ -22,17 +22,17 @@ MakeUniqueCheck::MakeUniqueCheck(StringRef Name,
 MakeUniqueCheck::SmartPtrTypeMatcher
 MakeUniqueCheck::getSmartPointerTypeMatcher() const {
   return qualType(hasDeclaration(classTemplateSpecializationDecl(
-      matchesName("::std::unique_ptr"), templateArgumentCountIs(2),
+      hasName("::std::unique_ptr"), templateArgumentCountIs(2),
       hasTemplateArgument(
           0, templateArgument(refersToType(qualType().bind(PointerType)))),
       hasTemplateArgument(
-          1, templateArgument(refersToType(
-                 qualType(hasDeclaration(classTemplateSpecializationDecl(
-                     matchesName("::std::default_delete"),
-                     templateArgumentCountIs(1),
-                     hasTemplateArgument(
-                         0, templateArgument(refersToType(qualType(
-                                equalsBoundNode(PointerType))))))))))))));
+          1,
+          templateArgument(refersToType(
+              qualType(hasDeclaration(classTemplateSpecializationDecl(
+                  hasName("::std::default_delete"), templateArgumentCountIs(1),
+                  hasTemplateArgument(
+                      0, templateArgument(refersToType(
+                             qualType(equalsBoundNode(PointerType))))))))))))));
 }
 
 } // namespace modernize
