@@ -93,7 +93,7 @@ const HeaderMapCollector::RegexHeaderMap *getSTLPostfixHeaderMap() {
       {"bits/basic_string.h$", "<string>"},
       {"bits/basic_string.tcc$", "<string>"},
       {"bits/char_traits.h$", "<string>"},
-      {"bits/codecvt.h$", "<fstream>"},
+      {"bits/codecvt.h$", "<locale>"},
       {"bits/concept_check.h$", "<numeric>"},
       {"bits/cpp_type_traits.h$", "<cmath>"},
       {"bits/cxxabi_forced.h$", "<cxxabi.h>"},
@@ -120,18 +120,18 @@ const HeaderMapCollector::RegexHeaderMap *getSTLPostfixHeaderMap() {
       {"bits/locale_facets.tcc$", "<locale>"},
       {"bits/locale_facets_nonio.h$", "<locale>"},
       {"bits/locale_facets_nonio.tcc$", "<locale>"},
-      {"bits/localefwd.h$", "<string>"},
+      {"bits/localefwd.h$", "<locale>"},
       {"bits/mask_array.h$", "<valarray>"},
-      {"bits/memoryfwd.h$", "<string>"},
+      {"bits/memoryfwd.h$", "<memory>"},
       {"bits/move.h$", "<utility>"},
       {"bits/nested_exception.h$", "<exception>"},
       {"bits/ostream.tcc$", "<ostream>"},
-      {"bits/ostream_insert.h$", "<string>"},
+      {"bits/ostream_insert.h$", "<ostream>"},
       {"bits/postypes.h$", "<iosfwd>"},
-      {"bits/ptr_traits.h$", "<unordered_set>"},
+      {"bits/ptr_traits.h$", "<memory>"},
       {"bits/random.h$", "<random>"},
       {"bits/random.tcc$", "<random>"},
-      {"bits/range_access.h$", "<string>"},
+      {"bits/range_access.h$", "<iterator>"},
       {"bits/regex.h$", "<regex>"},
       {"bits/regex_compiler.h$", "<regex>"},
       {"bits/regex_constants.h$", "<regex>"},
@@ -316,8 +316,8 @@ const HeaderMapCollector::RegexHeaderMap *getSTLPostfixHeaderMap() {
       {"bits/basic_file.h$", "<fstream>"},
       {"bits/c++allocator.h$", "<string>"},
       {"bits/c++config.h$", "<iosfwd>"},
-      {"bits/c++io.h$", "<fstream>"},
-      {"bits/c++locale.h$", "<string>"},
+      {"bits/c++io.h$", "<ios>"},
+      {"bits/c++locale.h$", "<locale>"},
       {"bits/cpu_defines.h$", "<iosfwd>"},
       {"bits/ctype_base.h$", "<locale>"},
       {"bits/cxxabi_tweaks.h$", "<cxxabi.h>"},
